@@ -26,6 +26,8 @@ import sys
 import os.path as op
 from pathlib import Path
 import json
+
+from nibabel.nifti2 import Nifti2Image
 from spec2nii import nifti_mrs
 from spec2nii import __version__ as spec2nii_ver
 # There are case specific imports below
@@ -36,8 +38,7 @@ class spec2nii:
         parser = argparse.ArgumentParser(description='Convert raw spectroscopy data to NIfTI format.')
         parser.add_argument('-v', '--version', action='version', version=spec2nii_ver)
 
-        subparsers = parser.add_subparsers(title='subcommands',
-                                           description='File types supported')
+        subparsers = parser.add_subparsers(title='spec2nii subcommands')
 
         def add_common_parameters(subparser):
             # Add options that are common to all subcommands
@@ -68,6 +69,7 @@ class spec2nii:
         for idx in range(5, 8):
             parser_twix.add_argument(f"-d{idx}", f"--dim{idx}", type=str, help=f"Specify dim {idx} loop counter.")
             parser_twix.add_argument(f"-t{idx}", f"--tag{idx}", type=str, help=f"Specify dim {idx} NIfTI MRS tag.")
+        parser_twix.add_argument('--remove_os', action='store_true', help='Remove time-domain oversampling.')
         parser_twix = add_common_parameters(parser_twix)
         parser_twix.set_defaults(func=self.twix)
 
@@ -76,9 +78,16 @@ class spec2nii:
         parser_dicom.add_argument('file', help='file or directory to convert', type=str)
         parser_dicom.add_argument("-t", "--tag", type=str, help="Specify NIfTI MRS tag used for 5th "
                                                                 "dimension if multiple files are passed.")
+        parser_dicom.add_argument('--voi', action='store_true', help='Output VOI as single voxel NIfTI mask.')
         parser_dicom = add_common_parameters(parser_dicom)
         parser_dicom.set_defaults(func=self.dicom)
 
+        # Handle rda subcommand
+        parser_dicom = subparsers.add_parser('rda', help='Convert from Siemens spectroscopy .rda format.')
+        parser_dicom.add_argument('file', help='file to convert', type=Path)
+        parser_dicom = add_common_parameters(parser_dicom)
+        parser_dicom.set_defaults(func=self.rda)
+
         # Handle UIH DICOM subcommand
         parser_uih_dicom = subparsers.add_parser('uih', help='Convert from UIH DICOM format.')
         parser_uih_dicom.add_argument('file', help='file or directory to convert', type=str)
@@ -91,8 +100,11 @@ class spec2nii:
         parser_philips = subparsers.add_parser('philips', help='Convert from Philips spar/sdat format.')
         parser_philips.add_argument('sdat', help='SDAT file', type=Path)
         parser_philips.add_argument('spar', help='SPAR file', type=Path)
-        parser_philips.add_argument("-t", "--tag", type=str, help="Specify NIfTI MRS tag used for 5th "
-                                                                  "dimension if multiple transients are present.")
+        parser_philips.add_argument(
+            "-t", "--tag",
+            type=str,
+            default="DIM_DYN",
+            help="Specify NIfTI MRS tag used for 5th dimension if multiple transients are present.")
         parser_philips = add_common_parameters(parser_philips)
         parser_philips.set_defaults(func=self.philips)
 
@@ -133,7 +145,7 @@ class spec2nii:
         parser_txt.add_argument("-i", "--imagingfreq", type=float,
                                 help="Imaging (central) frequency in MHz", required=True)
         parser_txt.add_argument("-b", "--bandwidth", type=float,
-                                help="Reciever bandwidth (sweepwidth) in Hz.", required=True)
+                                help="Receiver bandwidth (sweepwidth) in Hz.", required=True)
         parser_txt.add_argument("-n", "--nucleus", type=str,
                                 help="Nucleus string. e.g. 1H or 31P.", required=True)
         parser_txt.add_argument("-a", "--affine", type=str, help="NIfTI affine file", required=False, metavar='<file>')
@@ -181,6 +193,51 @@ class spec2nii:
         parser_varian = add_common_parameters(parser_varian)
         parser_varian.set_defaults(func=self.varian)
 
+        # Additional functions - anonymise and dump
+        parser_anon = subparsers.add_parser('anon', help='Anonymise existing NIfTI-MRS file.')
+        parser_anon.add_argument('file', help='file to anonymise', type=Path)
+        parser_anon.add_argument("-r", "--remove", type=str, metavar='KEY',
+                                 action='append',
+                                 help="Explicitly remove key. Argument may be repeated")
+        parser_anon.add_argument("-f", "--fileout", type=str,
+                                 help="Output file base name (default = input file name)")
+        parser_anon.add_argument("-o", "--outdir", type=Path,
+                                 help="Output location (default = .)", default='.')
+        parser_anon.add_argument('-v', '--verbose', action='store_true')
+        parser_anon.set_defaults(func=self.anon,
+                                 json=False,
+                                 nifti1=False,
+                                 override_nucleus=None,
+                                 override_frequency=None,
+                                 override_dwelltime=None)
+
+        parser_dump = subparsers.add_parser('dump', help='Dump contents of headers from existing NIfTI-MRS file.')
+        parser_dump.add_argument('file', help='NIfTI-MRS file', type=Path)
+        parser_dump.set_defaults(func=self.dump,
+                                 outdir=None,
+                                 nifti1=False)
+
+        parser_extract = subparsers.add_parser('extract', help='Generate json sidecar from existing NIfTI-MRS file.')
+        parser_extract.add_argument('file', help='NIFTI-MRS file', type=Path)
+        parser_extract.add_argument("-o", "--outdir", type=Path,
+                                    help="Output location (default: directory of input file)", default=None)
+        parser_extract.set_defaults(func=self.extract,
+                                    nifti1=False)
+
+        parser_insert = subparsers.add_parser('insert', help='Insert json formatted file into existing NIfTI-MRS file.')
+        parser_insert.add_argument('file', help='NIFTI-MRS file', type=Path)
+        parser_insert.add_argument('json_file', help='JSON file to insert', type=Path)
+        parser_insert.add_argument("-f", "--fileout", type=str,
+                                   help="Output file base name (default = input file name)")
+        parser_insert.add_argument("-o", "--outdir", type=Path,
+                                   help="Output location (default = .)", default='.')
+        parser_insert.set_defaults(func=self.insert,
+                                   nifti1=False,
+                                   json=False,
+                                   override_nucleus=None,
+                                   override_frequency=None,
+                                   override_dwelltime=None)
+
         if len(sys.argv) == 1:
             parser.print_usage(sys.stderr)
             sys.exit(1)
@@ -210,7 +267,7 @@ class spec2nii:
             print('No files to write.')
 
     def implement_overrides(self, args):
-        """Implement any command line overides for essential parameters."""
+        """Implement any command line overrides for essential parameters."""
         for nifti_mrs_img in self.imageOut:
             if args.override_dwelltime:
                 nifti_mrs_img.set_dwell_time(args.override_dwelltime)
@@ -279,7 +336,8 @@ class spec2nii:
                                                             args.evalinfo,
                                                             overrides,
                                                             args.quiet,
-                                                            args.verbose)
+                                                            args.verbose,
+                                                            remove_os=args.remove_os)
 
     # (Siemens) DICOM (.ima) format
     def dicom(self, args):
@@ -303,6 +361,20 @@ class spec2nii:
 
         self.imageOut, self.fileoutNames = multi_file_dicom(files_in, args.fileout, args.tag, args.verbose)
 
+        # VOI mask - TO DO extend to other formats
+        if args.voi:
+            import numpy as np
+            from nibabel import save
+            for img, f_out in zip(self.imageOut, self.fileoutNames):
+                out = self.outputDir / (f_out + '_voi.nii.gz')
+                save(Nifti2Image(np.ones((1, 1, 1), int), img.hdr_ext['VOI']), out)
+
+    # Siemens RDA (.rda) format
+    def rda(self, args):
+        """Siemens RDA format handler."""
+        from spec2nii.rda import convert_rda
+        self.imageOut, self.fileoutNames = convert_rda(args.file, args.fileout, args.verbose)
+
     # (UIH) DICOM (.dcm) format
     def uih_dicom(self, args):
         """UIH DICOM format handler."""
@@ -416,6 +488,26 @@ class spec2nii:
         from spec2nii.varian_importer import read_varian
         self.imageOut, self.fileoutNames = read_varian(args)
 
+    # Anonymise function
+    def anon(self, args):
+        from spec2nii.anonymise import anon_nifti_mrs
+        self.imageOut, self.fileoutNames = anon_nifti_mrs(args)
+
+    # Dump function
+    def dump(self, args):
+        from spec2nii.other import dump_headers
+        dump_headers(args)
+
+    # Extract function
+    def extract(self, args):
+        from spec2nii.other import extract_hdr_ext
+        extract_hdr_ext(args)
+
+    # Insert function
+    def insert(self, args):
+        from spec2nii.other import insert_hdr_ext
+        self.imageOut, self.fileoutNames = insert_hdr_ext(args)
+
 
 def main(*args):
     spec2nii(*args)
