@@ -2,12 +2,15 @@
 Author: William Clarke <william.clarke@ndcn.ox.ac.uk>
 Copyright (C) 2020 University of Oxford
 """
-import spec2nii.GSL.gslfunctions as GSL
+import re
+from os.path import basename
+from datetime import datetime
+
 import numpy as np
+
+import spec2nii.GSL.gslfunctions as GSL
 from spec2nii.dcm2niiOrientation.orientationFuncs import dcm_to_nifti_orientation
 from spec2nii import nifti_mrs
-from datetime import datetime
-from os.path import basename
 from spec2nii import __version__ as spec2nii_ver
 
 
@@ -32,9 +35,49 @@ defaults = {'vb': {'Col': 'time',
                    'Rep': 'DIM_DYN',
                    'Eco': 'DIM_EDIT'}}
 
+xa_product_seq = ['svs_se', 'svs_st', 'svs_slaser']
 
-def process_twix(twixObj, base_name_out, name_in, dataKey, dim_overides, quiet=False, verbose=False):
-    """Process a twix file. Identify type of MRS and then pass to the relavent function."""
+
+class IncompatibleSoftwareVersion(Exception):
+    pass
+
+
+def xa_or_vx(hdr):
+    """Determine siemens software type
+
+    :param hdr: mapVBVDHdr object
+    :type img: mapvbvd.read_twix_hdr.twix_hdr
+    :raises IncompatibleSoftwareVersion: Raised if not a VX or XA twix file
+    :return: String either 'xa' or 'vx'
+    :rtype: str
+    """
+    if re.search(r'syngo MR XA\d{2}', hdr['Dicom']['SoftwareVersions']):
+        return 'xa'
+    elif re.search(r'syngo MR [A-Z]\d{2}', hdr['Dicom']['SoftwareVersions']):
+        return 'vx'
+    else:
+        raise IncompatibleSoftwareVersion(
+            'spec2nii does not recognise this Siemens software version'
+            f' ({hdr["Dicom"]["SoftwareVersions"]}).'
+            ' spec2nii is tested on VA-VE, and XA20 and XA30 DICOM files.')
+
+
+def is_xa_product(hdr):
+    """If the basline is xa and the sequence is a svs prodcut sequence return True."""
+    def is_product():
+        seq_name = hdr['Meas'][('tSequenceString')]
+        if any([seq_name == seq for seq in xa_product_seq]):
+            return True
+
+    if xa_or_vx(hdr) == 'xa'\
+            and is_product():
+        return True
+    else:
+        return False
+
+
+def process_twix(twixObj, base_name_out, name_in, dataKey, dim_overides, quiet=False, verbose=False, remove_os=False):
+    """Process a twix file. Identify type of MRS and then pass to the relevant function."""
 
     if twixObj.hdr.Meas.lFinalMatrixSizePhase \
             and twixObj.hdr.Meas.lFinalMatrixSizeRead:
@@ -53,7 +96,15 @@ def process_twix(twixObj, base_name_out, name_in, dataKey, dim_overides, quiet=F
     if n_voxels > 1:
         return process_mrsi(twixObj, base_name_out, name_in, dataKey, quiet=quiet, verbose=verbose)
     else:
-        return process_svs(twixObj, base_name_out, name_in, dataKey, dim_overides, quiet=quiet, verbose=verbose)
+        return process_svs(
+            twixObj,
+            base_name_out,
+            name_in,
+            dataKey,
+            dim_overides,
+            remove_os,
+            quiet=quiet,
+            verbose=verbose)
 
 
 def process_mrsi(twixObj, base_name_out, name_in, dataKey, quiet=False, verbose=False):
@@ -61,18 +112,19 @@ def process_mrsi(twixObj, base_name_out, name_in, dataKey, quiet=False, verbose=
     raise NotImplementedError('MRSI pathway not yet implemented.')
 
 
-def process_svs(twixObj, base_name_out, name_in, dataKey, dim_overides, quiet=False, verbose=False):
+def process_svs(twixObj, base_name_out, name_in, dataKey, dim_overides, remove_os, quiet=False, verbose=False):
     """Process a twix file into a NIfTI MRS file.
     Inputs:
         twixObj: object from mapVBVD.
         base_name_out: Core string of output file.
         name_in: name of input file.
         dataKey: eval info flag name,
-        quire: True to suppress text output.
+        remove_os: Remove time-doain overrides
+        quiet: True to suppress text output.
     """
 
     # Set squeeze data
-    twixObj[dataKey].flagRemoveOS = False
+    twixObj[dataKey].flagRemoveOS = remove_os
     twixObj[dataKey].squeeze = True
     squeezedData = twixObj[dataKey]['']
 
@@ -107,11 +159,13 @@ def process_svs(twixObj, base_name_out, name_in, dataKey, dim_overides, quiet=Fa
 
     # Extract dwellTime
     dwellTime = twixObj['hdr']['MeasYaps'][('sRXSPEC', 'alDwellTime', '0')] / 1E9
+    if remove_os:
+        dwellTime *= 2
 
     # Extract metadata
     meta_obj = extractTwixMetadata(twixObj['hdr'], basename(twixObj[dataKey].filename))
 
-    # Identify what those indicies are
+    # Identify what those indices are
     # If cha is one: loop over 3rd and higher dims and make 2D images
     # If cha isn't present one: loop over 2nd and higher dims and make 1D images
     # Don't write here, just fill up class property lists for later writing
@@ -126,8 +180,34 @@ def process_svs(twixObj, base_name_out, name_in, dataKey, dim_overides, quiet=Fa
         raise ValueError('Col is expected to be the first dimension in the Twix file, it is not.')
 
     curr_defaults = defaults[twixObj[dataKey].softwareVersion]
-
     dim_order = twixObj[dataKey].sqzDims[1:]
+
+    # SPECIAL CASE FOR XA 20/30 product sequences.
+    # A single? reference scan is encoded in the first element of the phs loop
+    # If so strip out the scan and add it as a separate output file.
+    if is_xa_product(twixObj['hdr'])\
+            and 'Phs' in dim_order:
+
+        phs_dim = dim_order.index('Phs') + 1
+        ref_index = [slice(None), ] * squeezedData.ndim
+        ref_index[phs_dim] = slice(1)
+        xa_ref_scans = squeezedData[tuple(ref_index)]
+        ref_index[phs_dim] = slice(1, None, 1)
+        squeezedData_noref = squeezedData[tuple(ref_index)]
+
+        if squeezedData_noref.shape[phs_dim] == 1:
+            dim_order.remove('Phs')
+            squeezedData = squeezedData_noref.squeeze()
+
+        # To do, handle any other dimensions properly
+        xa_ref_scans = xa_ref_scans.squeeze()
+        xa_zero_index = xa_ref_scans[0, :, :] == 0.0
+        xa_ref_scans = xa_ref_scans[:, ~xa_zero_index]\
+            .reshape(xa_ref_scans.shape[0], xa_ref_scans.shape[1], -1)
+    else:
+        xa_ref_scans = None
+
+    # Make list of tags (both default and user specified)
     dim_tags = []
     unknown_counter = 0
     for do in dim_order:
@@ -158,9 +238,9 @@ def process_svs(twixObj, base_name_out, name_in, dataKey, dim_overides, quiet=Fa
             dim_tags[idx] = tag
 
     # Permute the order of dimension in the data
-    orignal = list(range(1, squeezedData.ndim))
+    original = list(range(1, squeezedData.ndim))
     new = [twixObj[dataKey].sqzDims.index(dd) for dd in dim_order]
-    reord_data = np.moveaxis(squeezedData, orignal, new)
+    reord_data = np.moveaxis(squeezedData, original, new)
 
     # Now assemble data
     nifit_mrs_out = []
@@ -200,6 +280,27 @@ def process_svs(twixObj, base_name_out, name_in, dataKey, dim_overides, quiet=Fa
 
             filename_out.append(out_name)
 
+    if xa_ref_scans is not None:
+        # Pad with three singleton dimensions (x,y,z)
+        newshape = (1, 1, 1) + xa_ref_scans.shape
+        if xa_ref_scans.ndim > 2:
+            ref_tags = ['DIM_COIL', 'DIM_DYN', None]
+        else:
+            ref_tags = ['DIM_COIL', None, None]
+
+        meta_obj_ref = extractTwixMetadata(twixObj['hdr'], basename(twixObj[dataKey].filename))
+        meta_obj_ref.set_standard_def('WaterSuppressed', True)
+
+        nifit_mrs_out.append(
+            assemble_nifti_mrs(
+                xa_ref_scans.reshape(newshape),
+                dwellTime,
+                orientation,
+                meta_obj_ref,
+                ref_tags))
+
+        filename_out.append(mainStr + '_ref')
+
     return nifit_mrs_out, filename_out
 
 
@@ -227,19 +328,33 @@ def twix2DCMOrientation(mapVBVDHdr, verbose=False):
         sliceThickness
 
     """
+    # Only single-slices are supported -- throw an error otherwise
+
+    if ('sGroupArray', 'asGroup', '0', 'nSize') in mapVBVDHdr['MeasYaps']:
+        nSlices = mapVBVDHdr['MeasYaps'][('sGroupArray', 'asGroup', '0', 'nSize')]
+        if nSlices != 1.0:
+            raise ValueError('In slice-selective spectroscopy, only the first slice is supported')
+
     # Orientation information
     if ('sSpecPara', 'sVoI', 'sNormal', 'dSag') in mapVBVDHdr['MeasYaps']:
         NormaldSag = mapVBVDHdr['MeasYaps'][('sSpecPara', 'sVoI', 'sNormal', 'dSag')]
+    elif ('sSliceArray', 'asSlice', '0', 'sNormal', 'dSag') in mapVBVDHdr['MeasYaps']:
+        # This is for slice-selective spectroscopy
+        NormaldSag = mapVBVDHdr['MeasYaps'][('sSliceArray', 'asSlice', '0', 'sNormal', 'dSag')]
     else:
         NormaldSag = 0.0
 
     if ('sSpecPara', 'sVoI', 'sNormal', 'dCor') in mapVBVDHdr['MeasYaps']:
         NormaldCor = mapVBVDHdr['MeasYaps'][('sSpecPara', 'sVoI', 'sNormal', 'dCor')]
+    elif ('sSliceArray', 'asSlice', '0', 'sNormal', 'dCor') in mapVBVDHdr['MeasYaps']:
+        NormaldCor = mapVBVDHdr['MeasYaps'][('sSliceArray', 'asSlice', '0', 'sNormal', 'dCor')]
     else:
         NormaldCor = 0.0
 
     if ('sSpecPara', 'sVoI', 'sNormal', 'dTra') in mapVBVDHdr['MeasYaps']:
         NormaldTra = mapVBVDHdr['MeasYaps'][('sSpecPara', 'sVoI', 'sNormal', 'dTra')]
+    elif ('sSliceArray', 'asSlice', '0', 'sNormal', 'dTra') in mapVBVDHdr['MeasYaps']:
+        NormaldTra = mapVBVDHdr['MeasYaps'][('sSliceArray', 'asSlice', '0', 'sNormal', 'dTra')]
     else:
         NormaldTra = 0.0
 
@@ -250,15 +365,22 @@ def twix2DCMOrientation(mapVBVDHdr, verbose=False):
 
     TwixSliceNormal = np.array([NormaldSag, NormaldCor, NormaldTra], dtype=float)
 
-    RoFoV = mapVBVDHdr['MeasYaps'][('sSpecPara', 'sVoI', 'dReadoutFOV')]
-    PeFoV = mapVBVDHdr['MeasYaps'][('sSpecPara', 'sVoI', 'dPhaseFOV')]
+    if ('sSliceArray', 'asSlice', '0', 'dReadoutFOV') in mapVBVDHdr['MeasYaps']:
+        RoFoV = mapVBVDHdr['MeasYaps'][('sSliceArray', 'asSlice', '0', 'dReadoutFOV')]
+        PeFoV = mapVBVDHdr['MeasYaps'][('sSliceArray', 'asSlice', '0', 'dPhaseFOV')]
+    else:
+        RoFoV = mapVBVDHdr['MeasYaps'][('sSpecPara', 'sVoI', 'dReadoutFOV')]
+        PeFoV = mapVBVDHdr['MeasYaps'][('sSpecPara', 'sVoI', 'dPhaseFOV')]
 
     dColVec_vector, dRowVec_vector = GSL.calc_prs(TwixSliceNormal, inplaneRotation, verbose)
 
     imageOrientationPatient = np.stack((dRowVec_vector, dColVec_vector), axis=0)
 
     pixelSpacing = np.array([PeFoV, RoFoV])  # [RoFoV PeFoV];
-    sliceThickness = mapVBVDHdr['MeasYaps'][('sSpecPara', 'sVoI', 'dThickness')]
+    if ('sSliceArray', 'asSlice', '0', 'dThickness') in mapVBVDHdr['MeasYaps']:
+        sliceThickness = mapVBVDHdr['MeasYaps'][('sSliceArray', 'asSlice', '0', 'dThickness')]
+    if ('sSpecPara', 'sVoI', 'dThickness') in mapVBVDHdr['MeasYaps']:
+        sliceThickness = mapVBVDHdr['MeasYaps'][('sSpecPara', 'sVoI', 'dThickness')]
 
     # Position info (including table position)
     if ('sSpecPara', 'sVoI', 'sPosition', 'dSag') in mapVBVDHdr['MeasYaps']:
@@ -295,9 +417,9 @@ def twix2DCMOrientation(mapVBVDHdr, verbose=False):
 
 
 def examineTwix(twixObj, fileName, mraid):
-    """ Print formated twix contents"""
+    """ Print formatted twix contents"""
 
-    print(f'Contents of file: {fileName}')
+    print(f'Contents of file {fileName}:')
 
     if isinstance(twixObj, list):
         print(f'Multiraid file, {len(twixObj)} files found.')
@@ -317,6 +439,14 @@ def examineTwix(twixObj, fileName, mraid):
 
 
 def extractTwixMetadata(mapVBVDHdr, orignal_file):
+    """Pass to appropriate extractTwixMetadata function for software version."""
+    if xa_or_vx(mapVBVDHdr) == 'vx':
+        return extractTwixMetadata_vx(mapVBVDHdr, orignal_file)
+    elif xa_or_vx(mapVBVDHdr) == 'xa':
+        return extractTwixMetadata_xa(mapVBVDHdr, orignal_file)
+
+
+def extractTwixMetadata_xa(mapVBVDHdr, orignal_file):
     """ Extract information from the pymapVBVD header to insert into the json sidecar.
 
     Args:
@@ -325,6 +455,118 @@ def extractTwixMetadata(mapVBVDHdr, orignal_file):
         obj (hdr_ext): NIfTI MRS hdr ext object.
     """
 
+    # Extract required metadata and create hdr_ext object
+    obj = nifti_mrs.hdr_ext(mapVBVDHdr['Dicom'][('lFrequency')] / 1E6,
+                            mapVBVDHdr['MeasYaps'][('sTXSPEC', 'asNucleusInfo', '0', 'tNucleus')].strip('"'))
+
+    # Standard defined metadata
+    # # 5.1 MRS specific Tags
+    # 'EchoTime'
+    obj.set_standard_def('EchoTime', mapVBVDHdr['Phoenix'][('alTE', '0')] * 1E-6)
+    # 'RepetitionTime'
+    tr = mapVBVDHdr['MeasYaps'][('alTR', '0')] / 1E6
+    obj.set_standard_def('RepetitionTime', float(tr))
+    # 'InversionTime'
+    if ('alTI', '0') in mapVBVDHdr['MeasYaps']:
+        obj.set_standard_def('InversionTime', float(mapVBVDHdr['MeasYaps'][('alTI', '0')]))
+    # 'MixingTime'
+    # 'ExcitationFlipAngle'
+    obj.set_standard_def('ExcitationFlipAngle', float(mapVBVDHdr['MeasYaps'][('adFlipAngleDegree', '0')]))
+    # 'TxOffset'
+    obj.set_standard_def('TxOffset', empty_str_to_0float(mapVBVDHdr['MeasYaps'][('sSpecPara', 'dDeltaFrequency')]))
+    # 'VOI'
+    # 'WaterSuppressed'
+    # TO DO
+    # 'WaterSuppressionType'
+    # 'SequenceTriggered'
+    # # 5.2 Scanner information
+    # 'Manufacturer'
+    obj.set_standard_def('Manufacturer', mapVBVDHdr['Dicom'][('Manufacturer')])
+    # 'ManufacturersModelName'
+    obj.set_standard_def('ManufacturersModelName', mapVBVDHdr['Dicom'][('ManufacturersModelName')])
+    # 'DeviceSerialNumber'
+    obj.set_standard_def('DeviceSerialNumber', str(mapVBVDHdr['Dicom'][('DeviceSerialNumber')]))
+    # 'SoftwareVersions'
+    obj.set_standard_def('SoftwareVersions', mapVBVDHdr['Dicom'][('SoftwareVersions')])
+    # 'InstitutionName'
+    obj.set_standard_def('InstitutionName', mapVBVDHdr['Dicom'][('InstitutionName')])
+    # 'InstitutionAddress'
+    obj.set_standard_def('InstitutionAddress', mapVBVDHdr['Dicom'][('InstitutionAddress')])
+    # 'TxCoil'
+    tx_coil_tuple = ('sCoilSelectMeas', 'aTxCoilSelectData', '0', 'asList', '0', 'sCoilElementID', 'tCoilID')
+    obj.set_standard_def('TxCoil', mapVBVDHdr['MeasYaps'][tx_coil_tuple])
+    # 'RxCoil'
+    rx_coil_1 = ('sCoilSelectMeas', 'aRxCoilSelectData', '0', 'asList', '0', 'sCoilElementID', 'tCoilID')
+    rx_coil_2 = ('asCoilSelectMeas', '0', 'asList', '0', 'sCoilElementID', 'tCoilID')
+    if rx_coil_1 in mapVBVDHdr['MeasYaps']:
+        obj.set_standard_def('RxCoil', mapVBVDHdr['MeasYaps'][rx_coil_1])
+    elif rx_coil_2 in mapVBVDHdr['MeasYaps']:
+        obj.set_standard_def('RxCoil', mapVBVDHdr['MeasYaps'][rx_coil_2])
+
+    # # 5.3 Sequence information
+    # 'SequenceName'
+    obj.set_standard_def('SequenceName', mapVBVDHdr['Meas'][('tSequenceString')])
+    # 'ProtocolName'
+    obj.set_standard_def('ProtocolName', mapVBVDHdr['Dicom'][('tProtocolName')])
+    # # 5.4 Sequence information
+    # 'PatientPosition'
+    obj.set_standard_def('PatientPosition', mapVBVDHdr['Meas'][('tPatientPosition')])
+    # 'PatientName'
+    obj.set_standard_def('PatientName', mapVBVDHdr['Meas'][('tPatientName')])
+    # 'PatientID'
+    # 'PatientWeight'
+    try:
+        obj.set_standard_def('PatientWeight', float(mapVBVDHdr['Meas'][('flUsedPatientWeight')]))
+    except ValueError:
+        pass
+    # 'PatientDoB'
+    obj.set_standard_def('PatientDoB', str(mapVBVDHdr['Meas'][('PatientBirthDay')]))
+    # 'PatientSex'
+    if mapVBVDHdr['Meas'][('PatientSex')] == 0.0:
+        sex_str = 'M'
+    elif mapVBVDHdr['Meas'][('PatientSex')] == 1.0:
+        sex_str = 'F'
+    elif mapVBVDHdr['Meas'][('PatientSex')] == 2.0:
+        sex_str = 'O'
+    else:
+        raise ValueError('Meas, PatientSex, should be 0, 1, or 2.')
+
+    obj.set_standard_def('PatientSex', sex_str)
+
+    # # 5.5 Provenance and conversion metadata
+    # 'ConversionMethod'
+    obj.set_standard_def('ConversionMethod', f'spec2nii v{spec2nii_ver}')
+    # 'ConversionTime'
+    conversion_time = datetime.now().isoformat(sep='T', timespec='milliseconds')
+    obj.set_standard_def('ConversionTime', conversion_time)
+    # 'OriginalFile'
+    obj.set_standard_def('OriginalFile', [orignal_file, ])
+    # # 5.6 Spatial information
+    # 'kSpace'
+    obj.set_standard_def('kSpace', [False, False, False])
+
+    # Some additional information
+    obj.set_user_def(key='PulseSequenceFile',
+                     value=mapVBVDHdr['Config'][('SequenceFileName')],
+                     doc='Sequence binary path.')
+    obj.set_user_def(key='IceProgramFile',
+                     value=mapVBVDHdr['Meas'][('tICEProgramName')],
+                     doc='Reconstruction binary path.')
+
+    return obj
+
+
+def extractTwixMetadata_vx(mapVBVDHdr, orignal_file):
+    """ Extract information from the pymapVBVD header to insert into the json sidecar.
+
+    Function Twix files from Numaris4 Vx software scanners.
+
+    Args:
+        dcmdata (dict): Twix headers
+    Returns:
+        obj (hdr_ext): NIfTI MRS hdr ext object.
+    """
+
     # Extract required metadata and create hdr_ext object
     obj = nifti_mrs.hdr_ext(mapVBVDHdr['Meas'][('Frequency')] / 1E6,
                             mapVBVDHdr['Meas'][('ResonantNucleus')])
@@ -385,7 +627,10 @@ def extractTwixMetadata(mapVBVDHdr, orignal_file):
     obj.set_standard_def('PatientName', mapVBVDHdr['Meas'][('PatientName')])
     # 'PatientID'
     # 'PatientWeight'
-    obj.set_standard_def('PatientWeight', mapVBVDHdr['Meas'][('flUsedPatientWeight')])
+    try:
+        obj.set_standard_def('PatientWeight', float(mapVBVDHdr['Meas'][('flUsedPatientWeight')]))
+    except ValueError:
+        pass
     # 'PatientDoB'
     obj.set_standard_def('PatientDoB', str(mapVBVDHdr['Meas'][('PatientBirthDay')]))
     # 'PatientSex'
